/*
 Simpel en recht toe recht aan programma om de Si5351A en ADF435x in te stellen op de gewenste frequenties.
 Si5351A Clk0 = 25 MHZ voor een LNB (Oudere octagon meestal 27 MHz)
 Si5351A Clk1 = 10 MHz voor de upconverter
 Si5351A Clk2 = 25 MHz als referentie voor het extra ADF435x bordje

 Een en ander moet nog wel netter gemaakt worden en kiesbaar via de optie jumpers maar voor nu werkt het.
 Andere frequenties of drive levels kunnen in de vershillend regels aangepast worden.
 De referentie voor de ADF435x op 25 MHz laten want de formules verwachten deze waarde.

 Veel dank gaat uit naar de makers van de 'Etherkit Si5351' library en oe6ocg voor zijn voorbeeld pragramma voor de ADF435x.
 Omdat dat beide stukjes GPL zijn is dit stukje broddelwerk dat ook.

 2019 - Ad Sekeris PE1GIJ

 Versie: 0.0.1 - 24-12-2019
*/
 
#include "si5351.h"   // Bibliotheek 'Etherkit Si5351'
#include "Wire.h"     // Bibliotheek voor I2C
#include "SPI.h"      // Bibliotheek voor SPI

Si5351 si5351;

const int slaveSelectPin = 10;  //SPI-SS om de ADF435x te selecteren

long Freq = 30750000;  //Start frequentie in 100Hz resolutie
long refin = 2500000; // Referentie = 25Mhz // Niet wijzigen !!!
long ChanStep = 10000; //Kanaal raster = 100Khz
unsigned long Reg[6]; //ADF435x Reg's

void setup()
{
  // Geef de 3 optie ingangen een pull-up weerstand
  pinMode(7, INPUT_PULLUP);  //JP7
  pinMode(8, INPUT_PULLUP);  //JP6
  pinMode(9, INPUT_PULLUP);  //JP5
  // Start de Si5351
  // Si5351A met extern clock signaal van 10 MHz, geen correctie
  si5351.init(SI5351_CRYSTAL_LOAD_0PF, 10000000, 0);
  // Uitgangs vermogen per kanaal zonder filtering
  // SI5351_DRIVE_2MA = 2mA in 50 Ohm ong. -1dBm
  // SI5351_DRIVE_2MA = 4mA in 50 Ohm ong. 5dBm
  // SI5351_DRIVE_2MA = 6mA in 50 Ohm ong. 7dBm
  // SI5351_DRIVE_2MA = 8mA in 50 Ohm ong. 8dBm
  si5351.drive_strength(SI5351_CLK0, SI5351_DRIVE_4MA);
  si5351.drive_strength(SI5351_CLK1, SI5351_DRIVE_4MA);
  si5351.drive_strength(SI5351_CLK2, SI5351_DRIVE_4MA);
  // Frequentie voor Clock0. Let op de 2 extra nullen (Referentie voor LNB)
  // Als je de regel in een remark veranderd blijft die uitgang uit
    si5351.set_freq(2500000000ULL, SI5351_CLK0); // meest gebruikt in moderne lnb's
  // Frequentie voor Clock1. Let op de 2 extra nullen (Referentie voor de Up-converter)
  si5351.set_freq(1000000000ULL, SI5351_CLK1);
  // Frequentie voor Clock2. Let op de 2 extra nullen (Referentie voor de ADF435x print)
  si5351.set_freq(2500000000ULL, SI5351_CLK2);

  si5351.update_status();

  pinMode (slaveSelectPin, OUTPUT);
  digitalWrite(slaveSelectPin, LOW);
  SPI.setDataMode(SPI_MODE0);
  SPI.setBitOrder(MSBFIRST);
  SPI.setClockDivider(SPI_CLOCK_DIV128);
  SPI.begin();
  delay(500);

  SetFreq(Freq);
}

void loop()
{
  // Na het instellen van de chips staat de arduino niets te doen
}

void SetFreq(long Freq)
{
  ConvertFreq(Freq, Reg);
  WriteADF2(5);
  delayMicroseconds(2500);
  WriteADF2(4);
  delayMicroseconds(2500);
  WriteADF2(3);
  delayMicroseconds(2500);
  WriteADF2(2);
  delayMicroseconds(2500);
  WriteADF2(1);
  delayMicroseconds(2500);
  WriteADF2(0);
  delayMicroseconds(2500);
}
void WriteADF2(int idx)
{ // make 4 byte from integer for SPI-Transfer
  byte buf[4];
  for (int i = 0; i < 4; i++)
    buf[i] = (byte)(Reg[idx] >> (i * 8));
  WriteADF(buf[3], buf[2], buf[1], buf[0]);
}
int WriteADF(byte a1, byte a2, byte a3, byte a4) {
  // write over SPI to ADF435x
  digitalWrite(slaveSelectPin, LOW);
  delayMicroseconds(10);
  SPI.transfer(a1);
  SPI.transfer(a2);
  SPI.transfer(a3);
  SPI.transfer(a4);
  Toggle();
}
int Toggle() {
  digitalWrite(slaveSelectPin, HIGH);
  delayMicroseconds(5);
  digitalWrite(slaveSelectPin, LOW);
}

void ConvertFreq(long freq, unsigned long R[])
{
  // PLL-Reg-R0         =  32bit
  // Registerselect        3bit
  // int F_Frac = 4;       // 12bit
  // int N_Int = 92;       // 16bit
  // reserved           // 1bit

  // PLL-Reg-R1         =  32bit
  // Registerselect        3bit
  //int M_Mod = 5;        // 12bit
  int P_Phase = 1;     // 12bit bij 2x12bit achter elkaar pow()-bug !!
  int Prescal = 0;     // 1bit werkt niet ???
  int PhaseAdj = 0;    // 1bit werkt ook niet ???
  // reserved           // 3bit

  // PLL-Reg-R2         =  32bit
  // Registerselect        3bit
  int U1_CountRes = 0; // 1bit
  int U2_Cp3state = 0; // 1bit
  int U3_PwrDown = 0;  // 1bit
  int U4_PDpola = 1;    // 1bit
  int U5_LPD = 0;       // 1bit
  int U6_LPF = 1;       // 1bit 1=Integer, 0=Fractioneel nog niet ondersteund
  int CP_ChgPump = 7;     // 4bit
  int D1_DoublBuf = 0; // 1bit
  //  int R_Counter = 1;   // 10bit
  //  int RD1_Rdiv2 = 0;    // 1bit
  //  int RD2refdoubl = 0; // 1bit
  int M_Muxout = 0;     // 3bit
  int LoNoisSpur = 0;      // 2bit
  // reserved           // 1bit

  // PLL-Reg-R3         =  32bit
  // Registerselect        3bit
  int D_Clk_div = 150; // 12bit
  int C_Clk_mode = 0;   // 2bit
  //  reserved          // 1bit
  int F1_Csr = 0;       // 1bit
  //  reserved          // 2bit
  int F2_ChgChan = 0;   // 1bit
  int F3_ADB = 0;       // 1bit
  int F4_BandSel = 0;  // 1bit
  //  reserved          // 8bit

  // PLL-Reg-R4         =  32bit
  // Registerselect        3bit
  int D_out_PWR = 0 ;    // 2bit -4dBm Zou genoeg moeten zijn voor de mixer
  int D_RF_ena = 1;     // 1bit
  int D_auxOutPwr = 0;  // 2bit
  int D_auxOutEna = 0;  // 1bit
  int D_auxOutSel = 0;  // 1bit
  int D_MTLD = 0;       // 1bit
  int D_VcoPwrDown = 0; // 1bit 1=VCO off

  //  int B_BandSelClk = 200; // 8bit 

  int D_RfDivSel = 3;    // 3bit 3=70cm 4=2m 
  int D_FeedBck = 1;     // 1bit             AANPASSEN
  // reserved           // 8bit

  // PLL-Reg-R5         =  32bit
  // Registerselect     // 3bit
  // reserved           // 16bit
  // reserved     11    // 2bit
  // reserved           // 1bit
  int D_LdPinMod = 1;    // 2bit moeten 1 zijn
  // reserved           // 8bit

  // Referenz Freg Calc
  //  long refin = 250000; // Refrentie = 25000000hz
  int R_Counter = 1;   // 10bit
  int RD1_Rdiv2 = 0;    // 1bit
  int RD2refdoubl = 0; // 1bit
  int B_BandSelClk = 200; // 8bit            AANPASSEN
  //  int F4_BandSel = 0;  // 1bit

  // int F4_BandSel = 10.0 * B_BandSelClk / PFDFreq;

  long RFout = Freq;   // VCO-Frequetie
  // calc bandselect und RF-div
  int outdiv = 1;

  if (RFout >= 220000000) {
    outdiv = 1;
    D_RfDivSel = 0;
  }
  if (RFout < 220000000) {
    outdiv = 2;
    D_RfDivSel = 1;
  }
  if (RFout < 110000000) {
    outdiv = 4;
    D_RfDivSel = 2;
  }
  if (RFout < 55000000) {
    outdiv = 8;
    D_RfDivSel = 3;
  }
  if (RFout < 27500000) {
    outdiv = 16;
    D_RfDivSel = 4;
  }
  if (RFout < 13750000) {
    outdiv = 32;
    D_RfDivSel = 5;
  }

  float PFDFreq = refin * ((1.0 + RD2refdoubl) / (R_Counter * (1.0 + RD1_Rdiv2))); //Referenzfrequentie
  float N = ((RFout) * outdiv) / PFDFreq;
  int N_Int = N;
  long M_Mod = PFDFreq * (100000 / ChanStep) / 100000;
  int F_Frac = round((N - N_Int) * M_Mod);

  R[0] = (unsigned long)(0 + F_Frac * pow(2, 3) + N_Int * pow(2, 15));
  R[1] = (unsigned long)(1 + M_Mod * pow(2, 3) + P_Phase * pow(2, 15) + Prescal * pow(2, 27) + PhaseAdj * pow(2, 28));
    //R[1] = (R[1])+1; // Registerselect adjust ?? because unpossible 2x12bit in pow() funktion
  R[2] = (unsigned long)(2 + U1_CountRes * pow(2, 3) + U2_Cp3state * pow(2, 4) + U3_PwrDown * pow(2, 5) + U4_PDpola * pow(2, 6) + U5_LPD * pow(2, 7) + U6_LPF * pow(2, 8) + CP_ChgPump * pow(2, 9) + D1_DoublBuf * pow(2, 13) + R_Counter * pow(2, 14) + RD1_Rdiv2 * pow(2, 24) + RD2refdoubl * pow(2, 25) + M_Muxout * pow(2, 26) + LoNoisSpur * pow(2, 29));
  R[3] = (unsigned long)(3 + D_Clk_div * pow(2, 3) + C_Clk_mode * pow(2, 15) + 0 * pow(2, 17) + F1_Csr * pow(2, 18) + 0 * pow(2, 19) + F2_ChgChan * pow(2, 21) +  F3_ADB * pow(2, 22) + F4_BandSel * pow(2, 23) + 0 * pow(2, 24));
  R[4] = (unsigned long)(4 + D_out_PWR * pow(2, 3) + D_RF_ena * pow(2, 5) + D_auxOutPwr * pow(2, 6) + D_auxOutEna * pow(2, 8) + D_auxOutSel * pow(2, 9) + D_MTLD * pow(2, 10) + D_VcoPwrDown * pow(2, 11) + B_BandSelClk * pow(2, 12) + D_RfDivSel * pow(2, 20) + D_FeedBck * pow(2, 23));
  R[5] = (unsigned long)(5 + 0 * pow(2, 3) + 3 * pow(2, 19) + 0 * pow(2, 21) + D_LdPinMod * pow(2, 22));
}
